// MUSIC 250a - Winter 2018, CCRMA, Stanford University
// Romain Michon & Ge Wang
// Description: A chuck program receiving MIDI CC messages from a Teensy
// microcontroller to change the parameters of a simple synth. This version
// implements interpolation between the different values.

// synth algorithm
SawOsc saw => dac;

// oscillator base frequency
50 => float baseFreq;

// the interpolator
vec3 i;
// the interpolation rate
2::ms => dur irate;
// set initial .value, .goal, .slew
i.set( baseFreq, baseFreq, .05 * (second/irate) );

// function to drive interpolator(s) concurrently
fun void interpolate( dur delta )
{
  while( true ){
    // interpolate oscillator frequency
    i.interp( delta ) => saw.freq;
    // advance time by rate
    delta => now;
  }
}

// spork interpolate function
spork ~ interpolate( irate );

// number of the device to open (see: chuck --probe)
1 => int device;
// get command line
if( me.args() ) me.arg(0) => Std.atoi => device;

// the midi event
MidiIn min;
// the message for retrieving data
MidiMsg msg;

// open the device
if( !min.open( device ) ) me.exit();

// print out device that was opened
<<< "MIDI device:", min.num(), " -> ", min.name() >>>;

// infinite time-loop
while( true ){
  // wait on the event 'min'
  min => now;

  // get the message(s)
  while( min.recv(msg) ){
    if(msg.data2 == 0){ // 0 is the MIDI CC number
      // formating sawtooth freq
      msg.data3*10 + baseFreq => i.update;
    }
  }
}
